==> WifiScanner-0.8.0.tar.gz: 0.8.0/src/netlink.c <==
/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * File : netlink.c
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * Base code was from prismstumbler Jan Fernquist <Jan.B.Fernquist@telia.com>
 * and wlanctl from www.linux-wlan.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: netlink.c,v 1.10 2002/12/26 15:52:05 poggij Exp $
 */

#include <include.h>
#include <src/netlink.h>

// TODO : Define TypeOfCard by command line
UINT8 TypeOfCard = WLAN_NG_CARD;

extern unsigned int DebugLevel;

#ifdef LWNG_14
static CaptureArg ca;
static char errbuf[PCAP_ERRBUF_SIZE];
#endif /* ifdef LWNG_14 */
#ifdef LWNG_13
static int RawSock;
#endif /* ifdef LWNG_13 */

// Internal funtions
int getPacketWLAN (unsigned char *buf, int maxlen, unsigned int timeout);


/*
 * Open the NETLINK Device
 */
#ifdef LWNG_14
int
openPacket (char *devname)
{
  int DataLink;

  ca.pcap = pcap_open_live (devname, 3000, 1, 0, errbuf);
  if (ca.pcap) {
    pcap_setnonblock (ca.pcap, 1, errbuf);
    DataLink = pcap_datalink (ca.pcap);
    if (DebugLevel >= 2) {
      fprintf (stderr, "pcap_datalink(ca.pcap) = %d\n", DataLink);
    }
    switch (DataLink) {
    case DLT_PRISM_HEADER:
      //printf ("DLT_PRISM_HEADER\n");
      ca.offset = 144;
      break;
    case DLT_IEEE802_11:
      //printf ("DLT_IEEE802_11\n");
      ca.offset = 0;
      break;
    case DLT_AIRONET_HEADER:
      //printf ("DLT_AIRONET_HEADER:\n");
      ca.offset = 0;
      break;
    default:			//COOKED
      //printf ("COOKED:\n");
      ca.offset = 160;
    }
    return 1;
  }
//  printf("Pcap_reer = %s\n",ca.pcap->errbuf);
  return -1;
}
#else
int
openPacket (void)
{
  struct sockaddr_nl nl_sk_addr;

  if ((RawSock = socket (PF_NETLINK, SOCK_RAW, MCAST_GRP_SNIFF)) < 0) {
    perror ("Could not create socket");
    fprintf (stderr, "You usually have to be root to open raw sockets\n");
    return -1;
  }

  memset (&nl_sk_addr, 0, sizeof (struct sockaddr_nl));
  nl_sk_addr.nl_family = (sa_family_t) PF_NETLINK;
  nl_sk_addr.nl_pid = (unsigned int) getpid ();
  nl_sk_addr.nl_groups = MCAST_GRP_SNIFF;

  if (bind (RawSock, (struct sockaddr *) &nl_sk_addr,
	    sizeof (struct sockaddr_nl)) < 0) {
    perror ("Call to bind failed");
    close (RawSock);
    return -1;
  }
  return RawSock;
}
#endif

/*
 * Just a wrapper....
 */
void
closePacket (void)
{
#ifdef LWNG_14
  pcap_close (ca.pcap);
#else
  close (RawSock);
#endif
}

/* 
 * Write the help when sniff is actif
 */
void
InteractiveHelp (void)
{
  printf ("Active Keys :\n");
  printf ("H for Help\n");
  printf ("R for report\n");
  printf ("Q for quitting\n");
}

/*
 * Check if data is avaliable 
 * Grab a packet from Interface
 * Timeout is in micro second (s)
 */
int
getPacketWLAN (unsigned char *buf, int maxlen, unsigned int timeout)
{
#ifdef LWNG_14
  struct pcap_pkthdr pktHdr;
  u_char *ret;
#endif
  fd_set rs;

#ifdef LWNG_13
  register int r;
  register UINT8 n;
  struct timeval tm;
  struct timeval *ptm;
#endif
  register int stop_sniffing = 0;

  FD_ZERO (&rs);
#ifdef LWNG_13
  FD_SET (RawSock, &rs);
#endif
  FD_SET (0, &rs);

#ifdef LWNG_13
  if (timeout > 0) {
    tm.tv_sec = timeout / 1000000;
    tm.tv_usec = (timeout % 1000000);
    ptm = &tm;
  } else {
    ptm = NULL;
  }
  r = select (RawSock + 1, &rs, NULL, NULL, ptm);

  if (r < 0) {
    if (errno != 4)		/* Don't write error if Interrupted system call */
      perror ("Select failed");
    return (-1);
  }
  if (r == 0) {
    if (timeout > 0) {
      // perror("Timeout"); 
      return (0);
    }
  }

  if (FD_ISSET (RawSock, &rs)) {
    r = recv (RawSock, buf, maxlen, 0);
    if (r < 0) {
      perror ("recv failed");
      return (-1);
    }
    return (r);
  }
#endif

#ifdef LWNG_14
  ret = (u_char *) pcap_next (ca.pcap, &pktHdr);
  if (ret) {
    memcpy (buf, ret, pktHdr.len);
    return pktHdr.len;
  } else {
    return 0;
  }
#endif

  return (stop_sniffing);	/* Noting to read */
}

// I's only a wrapper :-)
int
getPacket (unsigned char *buf, int maxlen, unsigned int timeout)
{
  int ret = 0;

  switch (TypeOfCard) {
  case WLAN_NG_CARD:
    ret = getPacketWLAN (buf, maxlen, timeout);
    break;
  case WLAN_NG_CARD_AVS_CFF2:
    perror ("Type of packet is not yet analysed");
    exit (EXIT_FAILURE);
    break;
  case CISCO_CARD:
    perror ("Type of packet is not yet analysed");
    exit (EXIT_FAILURE);
    break;
  case LUCENT_CARD:
    perror ("Type of packet is not yet analysed");
    exit (EXIT_FAILURE);
    break;
  default:
    perror ("Arrrrggg, type of packet is unknowned !!!");
    exit (EXIT_FAILURE);
    break;
  }
  return ret;
}

==> WifiScanner.0.9.0.tar.gz: 0.9.0/src/netlink.c <==
/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * File : netlink.c
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * Base code was from prismstumbler Jan Fernquist <Jan.B.Fernquist@telia.com>
 * and wlanctl from www.linux-wlan.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: netlink.c,v 1.13 2003/05/15 11:46:53 poggij Exp $
 */

#include <include.h>
#include <src/netlink.h>

// TODO : Define TypeOfCard by command line
UINT8 TypeOfCard = WLAN_NG_CARD;

extern unsigned int DebugLevel;

#ifdef LWNG_14
static CaptureArg ca;
static char errbuf[PCAP_ERRBUF_SIZE];
#endif /* ifdef LWNG_14 */
#ifdef LWNG_13
static int RawSock;
#endif /* ifdef LWNG_13 */

#ifdef LWNG_15
#ifdef ABSTRACTION_LAYER
p80211_caphdr_t wlan_header;
//UINT8 wlan_payload[WLAN_A4FR_MAXLEN];
UINT8 wlan_payload[MAX_BUFFER_SIZE];
#endif
#endif /* ifdef LWNG_13 */
// Internal funtions
int getPacketWLAN (unsigned char *buf, int maxlen, unsigned int timeout);


/*
 * Open the NETLINK Device
 */
#ifdef LWNG_14
int
openPacket (char *devname)
{
  int DataLink;

  ca.pcap = pcap_open_live (devname, 3000, 1, 0, errbuf);
  if (ca.pcap) {
    pcap_setnonblock (ca.pcap, 1, errbuf);
    DataLink = pcap_datalink (ca.pcap);
    if (DebugLevel >= 2) {
      fprintf (stderr, "pcap_datalink(ca.pcap) = %d\n", DataLink);
    }
    switch (DataLink) {
    case DLT_PRISM_HEADER:
      //printf ("DLT_PRISM_HEADER\n");
      ca.offset = 144;
      break;
    case DLT_IEEE802_11:
      //printf ("DLT_IEEE802_11\n");
      ca.offset = 0;
      break;
    case DLT_AIRONET_HEADER:
      //printf ("DLT_AIRONET_HEADER:\n");
      ca.offset = 0;
      break;
    default:			//COOKED
      //printf ("COOKED:\n");
      ca.offset = 160;
    }
    return 1;
  }
//  printf("Pcap_reer = %s\n",ca.pcap->errbuf);
  return -1;
}
#else
int
openPacket (void)
{
  struct sockaddr_nl nl_sk_addr;

  if ((RawSock = socket (PF_NETLINK, SOCK_RAW, MCAST_GRP_SNIFF)) < 0) {
    perror ("Could not create socket");
    fprintf (stderr, "You usually have to be root to open raw sockets\n");
    return -1;
  }

  memset (&nl_sk_addr, 0, sizeof (struct sockaddr_nl));
  nl_sk_addr.nl_family = (sa_family_t) PF_NETLINK;
  nl_sk_addr.nl_pid = (unsigned int) getpid ();
  nl_sk_addr.nl_groups = MCAST_GRP_SNIFF;

  if (bind (RawSock, (struct sockaddr *) &nl_sk_addr,
	    sizeof (struct sockaddr_nl)) < 0) {
    perror ("Call to bind failed");
    close (RawSock);
    return -1;
  }
  return RawSock;
}
#endif

/*
 * Just a wrapper....
 */
void
closePacket (void)
{
#ifdef LWNG_14
  pcap_close (ca.pcap);
#else
  close (RawSock);
#endif
}

/* 
 * Write the help when sniff is actif
 */
void
InteractiveHelp (void)
{
  printf ("Active Keys :\n");
  printf ("H for Help\n");
  printf ("R for report\n");
  printf ("Q for quitting\n");
}

/*
 * Check if data is avaliable 
 * Grab a packet from Interface
 * Timeout is in micro second (s)
 */
int
getPacketWLAN (unsigned char *buf, int maxlen, unsigned int timeout)
{
#ifdef LWNG_14
  struct pcap_pkthdr pktHdr;
  u_char *ret;
#endif
  fd_set rs;

#ifdef LWNG_13
  register int r;
  register UINT8 n;
  struct timeval tm;
  struct timeval *ptm;
#endif

  FD_ZERO (&rs);
#ifdef LWNG_13
  FD_SET (RawSock, &rs);
#endif
  FD_SET (0, &rs);

#ifdef LWNG_13
// DEPRECATED CODE !

  if (timeout > 0) {
    tm.tv_sec = timeout / 1000000;
    tm.tv_usec = (timeout % 1000000);
    ptm = &tm;
  } else {
    ptm = NULL;
  }
  r = select (RawSock + 1, &rs, NULL, NULL, ptm);

  if (r < 0) {
    if (errno != 4)		/* Don't write error if Interrupted system call */
      perror ("Select failed");
    return (-1);
  }
  if (r == 0) {
    if (timeout > 0) {
      debug (2, "Timeout");
      return (0);
    }
  }

  if (FD_ISSET (RawSock, &rs)) {
    r = recv (RawSock, buf, maxlen, 0);
    if (r < 0) {
      perror ("recv failed");
      return (-1);
    }
    return (r);
  }
#endif

#ifdef LWNG_14
  ret = (u_char *) pcap_next (ca.pcap, &pktHdr);
  // If no problem and packet is enought big (with data)
  if ((ret) && (pktHdr.len >= sizeof (p80211msg_lnxind_wlansniffrm_t))) {
    memcpy (buf, ret, pktHdr.len);
    return pktHdr.len;
  } else {
    return 0;
  }
#endif

  return (0);			/* Noting to read */
}

#ifdef ABSTRACTION_LAYER
int
ConvertPacketWLAN (unsigned char *buf, int maxlen, UINT16 recvlen)
{
  p80211msg_lnxind_wlansniffrm_t *Sniff_Frame;
  Sniff_Frame = (p80211msg_lnxind_wlansniffrm_t *) buf;
  // Fill Header
  wlan_header.version = 0;	// It's a reduced capture frame format
  wlan_header.length = 0;	// Not used for now
  wlan_header.mactime = Sniff_Frame->mactime.data;
  wlan_header.hosttime = Sniff_Frame->hosttime.data;
  wlan_header.phytype = phytype_dsss_dot11_b;	// Not used for now
  wlan_header.channel = Sniff_Frame->channel.data;
  wlan_header.datarate = Sniff_Frame->rate.data * 5;	// datarate is in units of 100kbps.
  wlan_header.antenna = 0;	// Not used for now
  wlan_header.priority = 0;	// Not used for now
  wlan_header.ssi_type = 0;	// Not used for now
  wlan_header.ssi_signal = Sniff_Frame->signal.data;
  wlan_header.ssi_noise = Sniff_Frame->noise.data;
  wlan_header.preamble = 0;	// Not used for now
  wlan_header.encoding = 0;	// Not used for now
  // Fill data frame
  memcpy (wlan_payload, &buf[sizeof (p80211msg_lnxind_wlansniffrm_t)],
	  recvlen - sizeof (p80211msg_lnxind_wlansniffrm_t));
  memcpy (buf, wlan_payload,
	  maxlen - sizeof (p80211msg_lnxind_wlansniffrm_t));
  return (recvlen - sizeof (p80211msg_lnxind_wlansniffrm_t));
  //return 0;
}
#endif

// I's only a wrapper :-)
int
getPacket (unsigned char *buf, int maxlen, unsigned int timeout)
{
  int ret = 0;
  switch (TypeOfCard) {
  case WLAN_NG_CARD:
    ret = getPacketWLAN (buf, maxlen, timeout);
#ifdef ABSTRACTION_LAYER
    if (ret != 0)
      ret = ConvertPacketWLAN (buf, maxlen, ret);
#endif
    break;
  case WLAN_NG_CARD_AVS_CFF2:
    perror ("Type of packet is not yet analysed");
    exit (EXIT_FAILURE);
    break;
  case CISCO_CARD:
    perror ("Type of packet is not yet analysed");
    exit (EXIT_FAILURE);
    break;
  case LUCENT_CARD:
    perror ("Type of packet is not yet analysed");
    exit (EXIT_FAILURE);
    break;
  default:
    perror ("Arrrrggg, type of packet is unknowned !!!");
    exit (EXIT_FAILURE);
    break;
  }
  return ret;
}
